/*
 * mini web server
 * Copyright 2014, Alex Sualdea.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */




#include "MiniWebServer.h"
#include "Settings.h"
#include <csignal>
#include <Poco/Event.h>
#include <Poco/File.h>

Poco::Event ctrl_c_event;

void signalHandler( int signum )
{
	ctrl_c_event.set();
}

int main(int argc, char* argv[])
{
	if (argc != 4 && argc != 3)
	{
		std::cout << argv[0] << " <port> <docroot> <listen only on loopback>" << std::endl;
		return 1;
	}
	Settings::get_instance().port = atoi(argv[1]);
	if (Settings::get_instance().port == 0)
	{
		std::cout << "Invalid port" << std::endl;
		return 1;
	}

	Settings::get_instance().docroot = argv[2];
	Poco::File docroot_file(Settings::get_instance().docroot);
	if (!docroot_file.exists() || !docroot_file.isDirectory())
	{
		std::cout << "Invalid document root" << std::endl;
		return 1;
	}

	if (argc == 3)
		Settings::get_instance().only_localhost = true;
	else
		Settings::get_instance().only_localhost = atoi(argv[3]) != 0;

	signal(SIGINT, signalHandler);
	MiniWebServer server;
	std::shared_ptr<MiniWebService> svc_sh(new WebShellService());
	server.addService(WebShellService::name(), svc_sh);
	server.start(Settings::get_instance().port, Settings::get_instance().only_localhost);
	std::cout 	<< argv[0] << " listening on " << (Settings::get_instance().only_localhost? "127.0.0.1":"0.0.0.0") << ":"
				<< Settings::get_instance().port << std::endl;

	// wait until ctrl-c (or windows service stop)
	ctrl_c_event.wait();

	server.stop();
	std::cout << "server stopped" << std::endl;

	return 0;
}



